from collections import MutableMapping
import logging
import hashlib
import urllib

import logger

import requests


class YaPush(MutableMapping, object):
    def __init__(self, message, loglevel="INFO", logfile=None):
        self.message = message
        self._parsed = {}
        self._verified = None

        global log

        if "log" not in globals():
            if logfile is not None:
                log = logging.getLogger(__name__)
                log.addHandler(logger.FileHandler(logfile))
                log.setLevel(getattr(logging, loglevel))
            else:
                log = logging.getLogger(__name__)
                log.addHandler(logger.StreamHandler())
                log.setLevel(getattr(logging, loglevel))

    # Collection methods start

    def __getitem__(self, key):
        return self._parsed[key]

    def __setitem__(self, key, value):
        self._parsed[key] = value

    def __delitem__(self, key):
        del self._parsed[key]

    def __iter__(self):
        return iter(self._parsed)

    def __len__(self):
        return len(self._parsed)

    # Collection methods end

    def __repr__(self):
        return "Push: {}".format(self._parsed)

    def to_str(self):
        return "{}".format(self._parsed)

    def from_str(self, push_str):
        self._parsed = eval(push_str)
        return self._parsed

    @property
    def verified(self):
        return self._verified

    @verified.setter
    def verified(self, value):
        pass

    @verified.deleter
    def verified(self):
        del self._verified

    @property
    def parsed(self):
        return self._parsed

    @parsed.setter
    def parsed(self, value):
        pass

    @parsed.deleter
    def parsed(self):
        del self._parsed

    def parse(self, message=None):
        if message is not None:
            self.message = message
        smessage = self.message.split("&")
        result = {}

        for el in smessage:
            k, v = el.split("=")
            result[k] = v

        self._parsed = result

        return self._parsed

    def verify(self, push_secret):
        params = [
            self._parsed["notification_type"],
            self._parsed["operation_id"], self._parsed["amount"],
            self._parsed["currency"], self._parsed["datetime"],
            self._parsed["sender"], self._parsed["codepro"],
            push_secret, self._parsed["label"]
        ]

        if "unaccepted" in self._parsed and \
                self._parsed["unaccepted"] == "true":
            log.warning("Payment failed to accept! UNACCEPTED")
        if self._parsed["codepro"] == "true":
            log.warning("Payment failed to accept! CODEPRO")

        check_str = "&".join(params)
        check_str = urllib.unquote_plus(check_str).decode("utf8")
        log.debug("Check string: {}".format(check_str))
        check_sum = hashlib.sha1(check_str).hexdigest()

        if check_sum == self._parsed["sha1_hash"]:
            log.info("Push verified")
            self._verified = True
        else:
            log.warning(
                "Push aren't verified\nChecksum:\t{}\nControlled:\t{}".
                format(check_sum, self._parsed["sha1_hash"])
            )
            self._verified = False

        return self.verified

    def get_details(self, token):
        headers = {"Authorization": "Bearer {}".format(token)}
        data = {"operation_id": self._parsed["operation_id"]}

        r = requests.post(
            "https://money.yandex.ru/api/operation-details",
            headers=headers, data=data
        )

        if r.status_code == 200:
            log.info("Got details successfully")
            self._parsed.update(r.json())
            return r.json()
        else:
            log.error("Can't get details. Reason: %s" % (r.reason))



import logging
import traceback
import urllib

import logger

import requests


class YaAuth(object):
    def __init__(self, redirect_url, client_id, client_secret,
                 push_secret, scope=["operation-details"],
                 base_url="https://sp-money.yandex.ru/oauth",
                 loglevel="INFO", logfile=None):
        self.base_url = base_url
        self.redirect_url = redirect_url
        self.client_id = client_id
        self.client_secret = client_secret
        self.push_secret = push_secret
        self.scope = scope

        global log

        if "log" not in globals():
            if logfile is not None:
                log = logging.getLogger(__name__)
                log.addHandler(logger.FileHandler(logfile))
                log.setLevel(getattr(logging, loglevel))
            else:
                log = logging.getLogger(__name__)
                log.addHandler(logger.StreamHandler())
                log.setLevel(getattr(logging, loglevel))

    def get_auth_url(self):
        try:
            auth_url = \
                self.base_url + "/authorize?" + "&".join(
                    [
                        "redirect_uri={}".format(
                            urllib.quote_plus(self.redirect_url)),
                        "scope={}".format("+".join(self.scope)),
                        "client_id={}".format(self.client_id),
                        "response_type=code"
                    ]
                )
            log.info(
                "\n### Auth URL ###\n\n{}\n\n###############".
                format(auth_url)
            )
            return auth_url
        except:
            log.error("Can't get auth URL for{}\n{}".format(
                self.client_id, traceback.format_exc()))

    def get_token(self, code):
        data = {
            "grant_type": "authorization_code",
            "code": code,
            "client_id": self.client_id,
            "client_secret": self.client_secret,
            "redirect_uri": self.redirect_url
        }

        r = requests.post(self.base_url + "/token", data=data)

        if r.status_code == 200:
            log.info("Access token has been gotten successfully")
            log.debug(r.json()["access_token"])
            return r.json()["access_token"]
        else:
            log.error("Can't get access token for %s\nReason: %s" % (
                self.client_id, r.reason))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
from BaseHTTPServer import HTTPServer, BaseHTTPRequestHandler
import logging
import ssl
import traceback
import urlparse
import yaml

from io import BytesIO

from minepycessor_libs import YaAuth
from minepycessor_libs import QueueBus, DBBus
from minepycessor_libs import logger


class AuthServer:
    def __init__(self, auth, qbus, dbus, conf):
        def handler(*args, **kwargs):
            SimpleHTTPRequestHandler(auth, qbus, dbus, conf, *args, **kwargs)
        httpd = HTTPServer((conf["address"], conf["port"]), handler)
        httpd.socket = ssl.wrap_socket(
            httpd.socket,
            keyfile=conf["keyfile"],
            certfile=conf["certfile"],
            server_side=True
        )
        httpd.serve_forever()


class SimpleHTTPRequestHandler(BaseHTTPRequestHandler):
    def __init__(self, auth, qbus, dbus, conf, *args, **kwargs):
        self.auth = auth
        self.qbus = qbus
        self.dbus = dbus
        self.conf = conf
        BaseHTTPRequestHandler.__init__(self, *args, **kwargs)

    def do_GET(self):
        self.send_response(200)
        self.end_headers()
        self.wfile.write(b"Thank you for choosing Giga-Ruby!")
        if "?" in self.path:
            path, tmp = self.path.split("?", 1)
            qs = urlparse.parse_qs(tmp)
            try:
                token = self.auth.get_token(qs["code"])
                self.dbus.put_token(
                    self.conf["database"]["token_name"],
                    token,
                    self.conf["database"]["token_table"]
                )
            except:
                log.error("Can't get access token/n{}".format(
                    traceback.format_exc()))

    def do_POST(self):
        content_length = int(self.headers["Content-Length"])
        body = self.rfile.read(content_length)
        self.send_response(200)
        self.end_headers()
        response = BytesIO()
        response.write(b"This is POST request. ")
        response.write(b"Received: ")
        response.write(body)
        log.debug("Body:\n{}".format(body))
        self.wfile.write(response.getvalue())
        try:
            self.qbus.put_push(body, self.conf["mqueue"]["queue"])
        except:
            log.error("Can't parse push \n{}".format(
                traceback.format_exc()))


def main():
    try:
        parser = argparse.ArgumentParser()

        parser.add_argument(
            "conf",
            help="Path to .yml file with client id and scopes"
        )
        parser.add_argument(
            "-a", "--address", dest="address",
            help="Name of the server"
        )
        parser.add_argument(
            "-p", "--port", dest="port",
            help="Port of http info server"
        )
        parser.add_argument(
            "-r", "--redirect", dest="redirect_url",
            help="Redirect URL for getting token"
        )
        parser.add_argument(
            "-k", "--keyfile", dest="keyfile",
            help="Path to keyfile"
        )
        parser.add_argument(
            "-c", "--certfile", dest="certfile",
            help="Path to certfile"
        )
        parser.add_argument(
            "-s", "--screen", dest="screen_name",
            help="Name of the server's screen"
        )
        parser.add_argument(
            "--loglevel", dest="loglevel",
            choices=[
                "CRITICAL", "ERROR", "WARNING",
                "INFO", "DEBUG", "NOTSET"
            ],
            help="Logging level"
        )
        parser.add_argument(
            "-l", "--log", dest="log",
            help="Redirect logging to file"
        )
        args = parser.parse_args()

        with open(args.conf, "r") as f:
            conf = yaml.load(f)

        if args.address is not None:
            conf["address"] = args.address
        if args.port is not None:
            conf["port"] = args.port
        if args.redirect_url is not None:
            conf["redirect_url"] = args.redirect_url
        if args.keyfile is not None:
            conf["keyfile"] = args.keyfile
        if args.certfile is not None:
            conf["certfile"] = args.certfile
        if args.screen_name is not None:
            conf["screen_name"] = args.screen_name
        if args.loglevel is not None:
            conf["loglevel"] = args.loglevel
        if args.log is not None:
            conf["logfile"] = args.log
        if conf["logfile"] == "":
            conf["logfile"] = None

        global log

        if args.log is not None:
            log = logging.getLogger(__name__)
            log.addHandler(logger.FileHandler(args.log))
            log.setLevel(getattr(logging, conf["loglevel"]))
        else:
            log = logging.getLogger(__name__)
            log.addHandler(logger.StreamHandler())
            log.setLevel(getattr(logging, conf["loglevel"]))

        auth = YaAuth(
            conf["redirect_url"],
            conf["client_id"],
            conf["client_secret"],
            conf["push_secret"],
            scope=conf["scope"],
            loglevel=conf["loglevel"],
            logfile=conf["logfile"]
        )
        auth.get_auth_url()
        qbus = QueueBus(
            conf["mqueue"]["host"],
            conf["mqueue"]["user"],
            conf["mqueue"]["password"],
            loglevel=conf["loglevel"],
            logfile=conf["logfile"]
        )
        qbus.connect()
        dbus = DBBus(
            conf["database"]["host"],
            conf["database"]["user"],
            conf["database"]["password"],
            conf["database"]["db"],
            loglevel=conf["loglevel"],
            logfile=conf["logfile"]
        )
        dbus.connect()
        dbus.create_token_table(conf["database"]["token_table"])

        httpd = AuthServer(auth, qbus, dbus, conf)

    except KeyboardInterrupt:
        print("\nThe process was interrupted by the user")
        raise SystemExit


if __name__ == "__main__":
    main()



import logging
import re
import subprocess
import traceback
import time

from minepycessor_libs import logger
from minepycessor_libs import YaPush


def loop(func):
    def wrapper(self, *args, **kwargs):
        if self.timeout is not None:
            while True:
                func(self, *args, **kwargs)
                time.sleep(self.timeout)
        else:
            func(self, *args, **kwargs)
        return
    return wrapper


class YaProcessor(object):
    def __init__(self, conf, token, timeout=5, loglevel="INFO", logfile=None):
        self.conf = conf
        self.token = token
        self.timeout = timeout

        global log

        if "log" not in globals():
            if logfile is not None:
                log = logging.getLogger(__name__)
                log.addHandler(logger.FileHandler(logfile))
                log.setLevel(getattr(logging, loglevel))
            else:
                log = logging.getLogger(__name__)
                log.addHandler(logger.StreamHandler())
                log.setLevel(getattr(logging, loglevel))

    @classmethod
    def parse_message(cls, message):
        sp = message.split(";")
        res = [sp[0].strip()]
        if len(sp) > 1:
            res.append(sp[1].strip())
        else:
            res.append(None)

        return res

    def perform_command(self, nickname, target):
        try:
            subprocess.call(
                [
                    "screen", "-x", self.conf["screen_name"],
                    "-p", "0", "-X", "stuff",
                    "{}^M".format(
                        re.sub(
                            "{{ name }}", nickname,
                            self.conf["menu"][target]["command"]
                        )
                    )
                ]
            )
        except:
            log.error("Can't perform command {}\n{}".format(
                target, traceback.format_exc()))

    def process_msg(self, message):
        push = YaPush(message)
        push.parse()
        push.verify(self.conf["push_secret"])
        push.get_details(self.token)
        log.info(
            "\n###################\n\n{}\n\n###################".
            format(push)
        )

        if "message" in push:
            target, nickname = self.parse_message(push["message"])
            if nickname is None:
                log.warning(
                    "Command hasn't been performed."
                    "Comment is empty for {}".format(push["sender"])
                )
            else:
                if target in self.conf["menu"]:
                    if float(push["withdraw_amount"]) == \
                            float(self.conf["menu"][target]["price"]):
                        self.perform_command(nickname, target)
                    else:
                        log.warning(
                            "Command hasn't been performed."
                            "Payment desn't correspond to price. {}".format(
                                push["withdraw_amount"])
                        )
                else:
                    log.warning(
                        "Command hasn't been performed."
                        "No such target in menue. {}".format(target)
                    )
        else:
            log.warning(
                "Something wrong with push, there is no 'message' field"
            )

    @loop
    def process(self, qbus):
        push_str = qbus.get_push(self.conf["mqueue"]["queue"])
        if push_str is not None:
            self.process_msg(push_str)
